use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
ARRAYn
AllUtils
BLOCK's
Champoux
DROLSKY
DROLSKY's
Dave
EXPR
Etheridge
Haskell
KEYFUNC
KVLIST
Karen
List
PayPal
Ricardo
Rolsky
Rolsky's
Signes
Yanick
autarch
bsearchidx
drolsky
ether
firstidx
firstres
firstval
lastidx
lastres
lastval
lib
maxstr
minmax
minstr
natatime
notall
onlyidx
onlyres
onlyval
pairfirst
pairgrep
pairkeys
pairmap
pairvalues
probability
randomisation
rjbs
specialisations
thusly
uniq
unselected
util
yanick
